package de.jottyfan.bico.modules.camp;

import java.security.Principal;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.jottyfan.bico.modules.camp.model.RegistrationBean;
import jakarta.validation.Valid;

/**
 *
 * @author jotty
 *
 */
@Service
public class RegistrationService {

	@Autowired
	private RegistrationRepository repository;

	/**
	 * save the bean content
	 *
	 * @param bean the bean
	 */
	public void save(@Valid RegistrationBean bean, Principal principal) {
		repository.save(bean, principal.getName());
	}

	/**
	 * get the registrations of principal
	 *
	 * @param principal the principal
	 * @return the registrations
	 */
	public List<RegistrationBean> getRegistrations(Principal principal) {
		return repository.getRegistrations(principal.getName());
	}

	/**
	 * get the registration if the principal was the creator
	 *
	 * @param id the ID of the registration
	 * @param principal the principal
	 * @return the bean or null if not found or allowed
	 */
	public RegistrationBean getBeanOfPrincipal(Integer id, Principal principal) {
		return repository.getBean(id, principal.getName());
	}

	/**
	 * delete the registration if the principal was the creator
	 *
	 * @param id the ID of the registration
	 * @param principal the principal
	 */
	public void delete(Integer id, Principal principal) {
		repository.delete(id, principal.getName());
	}
}
