package de.jottyfan.bico.modules.theme;

import static de.jottyfan.bico.db.public_.Tables.T_LESSON;
import static de.jottyfan.bico.db.public_.Tables.T_LESSON_SUBJECT;
import static de.jottyfan.bico.db.public_.Tables.T_SLOT;
import static de.jottyfan.bico.db.public_.Tables.T_SOURCE;
import static de.jottyfan.bico.db.public_.Tables.T_SUBJECT;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jboss.logging.Logger;
import org.jooq.DSLContext;
import org.jooq.DeleteConditionStep;
import org.jooq.InsertOnDuplicateStep;
import org.jooq.Record1;
import org.jooq.Record10;
import org.jooq.Record4;
import org.jooq.SelectConditionStep;
import org.jooq.SelectSeekStep1;
import org.jooq.UpdateConditionStep;
import org.jooq.impl.DSL;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.bico.db.public_.tables.records.TLessonRecord;
import de.jottyfan.bico.db.public_.tables.records.TLessonSubjectRecord;
import de.jottyfan.bico.modules.theme.model.KeyValueBean;
import de.jottyfan.bico.modules.theme.model.ThemeBean;

/**
 *
 * @author jotty
 *
 */
@Repository
public class ThemeRepository {
	private static final Logger LOGGER = Logger.getLogger(ThemeRepository.class);

	@Autowired
	private DSLContext jooq;

	public List<ThemeBean> getThemeManipulation(Integer slotId) {
		SelectConditionStep<Record10<String, String, String, String, String, String, String, Integer, String, Integer>> sql = jooq
		// @formatter:off
			.select(T_SUBJECT.THEME, T_SUBJECT.SUBTHEME, T_SUBJECT.BOOK_PAGES, T_SUBJECT.WORKSHEETS, T_SUBJECT.BIBLEVERSE, T_SUBJECT.NOTES,
					    T_SOURCE.NAME,
					    T_LESSON.PK_LESSON, T_LESSON.NOTES,
					    T_LESSON_SUBJECT.PK_LESSON_SUBJECT)
			.from(T_LESSON)
			.leftJoin(T_LESSON_SUBJECT).on(T_LESSON_SUBJECT.FK_LESSON.eq(T_LESSON.PK_LESSON))
			.leftJoin(T_SUBJECT).on(T_SUBJECT.PK_SUBJECT.eq(T_LESSON_SUBJECT.FK_SUBJECT))
			.leftJoin(T_SOURCE).on(T_SOURCE.PK_SOURCE.eq(T_SUBJECT.FK_SOURCE))
			.where(T_LESSON.FK_SLOT.eq(slotId));
		// @formatter:on
		LOGGER.trace(sql);
		Iterator<Record10<String, String, String, String, String, String, String, Integer, String, Integer>> i = sql.fetch()
				.iterator();
		List<ThemeBean> list = new ArrayList<>();
		while (i.hasNext()) {
			Record10<String, String, String, String, String, String, String, Integer, String, Integer> r = i.next();
			ThemeBean bean = ThemeBean.of(r.get(T_LESSON.PK_LESSON));
			bean.setSlotId(slotId);
			bean.setPkLessonSubject(r.get(T_LESSON_SUBJECT.PK_LESSON_SUBJECT));
			bean.setTheme(r.get(T_SUBJECT.THEME));
			bean.setSubtheme(r.get(T_SUBJECT.SUBTHEME));
			bean.setBookPages(r.get(T_SUBJECT.BOOK_PAGES));
			bean.setWorksheets(r.get(T_SUBJECT.WORKSHEETS));
			bean.setBibleverse(r.get(T_SUBJECT.BIBLEVERSE));
			bean.setNotes(r.get(T_SUBJECT.NOTES));
			bean.setSourceName(r.get(T_SOURCE.NAME));
			bean.setLessonNotes(r.get(T_LESSON.NOTES));
			list.add(bean);
		}
		return list;
	}

	public List<KeyValueBean> getAllThemes() {
		SelectSeekStep1<Record4<Integer, String, String, String>, Integer> sql = jooq
		// @formatter:off
			.select(T_SUBJECT.PK_SUBJECT, T_SUBJECT.THEME, T_SUBJECT.SUBTHEME, T_SOURCE.NAME)
			.from(T_SUBJECT)
			.leftJoin(T_SOURCE).on(T_SOURCE.PK_SOURCE.eq(T_SUBJECT.FK_SOURCE))
			.orderBy(T_SUBJECT.ORDER_NR);
		// @formatter:on
		LOGGER.trace(sql);
		List<KeyValueBean> list = new ArrayList<>();
		Iterator<Record4<Integer, String, String, String>> i = sql.fetch().iterator();
		while (i.hasNext()) {
			Record4<Integer, String, String, String> r = i.next();
			StringBuilder buf = new StringBuilder();
			buf.append(r.get(T_SOURCE.NAME)).append(": ");
			buf.append(r.get(T_SUBJECT.THEME)).append(", ");
			buf.append(r.get(T_SUBJECT.SUBTHEME));
			list.add(KeyValueBean.of(r.get(T_SUBJECT.PK_SUBJECT), buf.toString()));
		}
		return list;
	}

	public void addThemeToSlot(Integer slotId, Integer pkSubject) {
		InsertOnDuplicateStep<TLessonSubjectRecord> sql = jooq
		// @formatter:off
			.insertInto(T_LESSON_SUBJECT, T_LESSON_SUBJECT.FK_LESSON, T_LESSON_SUBJECT.FK_SUBJECT)
			.select(jooq
				.select(T_LESSON.PK_LESSON, DSL.val(pkSubject))
				.from(T_LESSON)
				.where(T_LESSON.FK_SLOT.eq(slotId))
			);
		// @formatter:on
		LOGGER.trace(sql);
		sql.execute();
	}

	public void removeThemeFromSlot(Integer pkLessonSubject) {
		DeleteConditionStep<TLessonSubjectRecord> sql = jooq.deleteFrom(T_LESSON_SUBJECT)
				.where(T_LESSON_SUBJECT.PK_LESSON_SUBJECT.eq(pkLessonSubject));
		LOGGER.trace(sql);
		sql.execute();
	}

	public TLessonRecord getLesson(Integer slotId) {
		SelectConditionStep<TLessonRecord> sql = jooq
		// @formatter:off
			.selectFrom(T_LESSON)
			.where(T_LESSON.FK_SLOT.eq(slotId));
		// @formatter:on
		LOGGER.trace(sql);
		return sql.fetchOne();
	}

	public void updateLesson(TLessonRecord bean) {
		UpdateConditionStep<TLessonRecord> sql = jooq
		// @formatter:off
			.update(T_LESSON)
			.set(T_LESSON.NOTES, bean.getNotes())
			.where(T_LESSON.PK_LESSON.eq(bean.getPkLesson()));
		// @formatter:on
		LOGGER.trace(sql);
		sql.execute();
	}

	public LocalDate getSlotDay(Integer slotId) {
		SelectConditionStep<Record1<LocalDate>> sql = jooq
		// @formatter:off
			.select(T_SLOT.SLOT_DAY)
			.from(T_SLOT)
			.where(T_SLOT.PK_SLOT.eq(slotId));
		// @formatter:on
		LOGGER.trace(sql);
		return sql.fetchOne().get(T_SLOT.SLOT_DAY);
	}

}
